﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;

namespace HeuristicLab.Encodings.DecisionList {
  [Item("DecisionListCrossover", "Description missing")]
  [StorableClass]
  public abstract class DecisionListCrossover : SingleSuccessorOperator, IDecisionListCrossover {

    #region Parameter Properties
    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<ItemArray<DecisionList>> ParentsParameter {
      get { return (ScopeTreeLookupParameter<DecisionList>)Parameters["Parents"]; }
    }
    public ILookupParameter<DecisionList> ChildParameter {
      get { return (ILookupParameter<DecisionList>)Parameters["Child"]; }
    }
    #endregion

    [StorableConstructor]
    protected DecisionListCrossover(bool deserializing) : base(deserializing) { }
    protected DecisionListCrossover(DecisionListCrossover original, Cloner cloner)
      : base(original, cloner) {
    }
    public DecisionListCrossover()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator which should be used for stochastic crossover operators."));
      Parameters.Add(new ScopeTreeLookupParameter<DecisionList>("Parents", "The parent vectors which should be crossed."));
      ParentsParameter.ActualName = "DecisionList";
      Parameters.Add(new LookupParameter<DecisionList>("Child", "The child vector resulting from the crossover."));
      ChildParameter.ActualName = "DecisionList";
    }

    public override IOperation Apply() {
      ChildParameter.ActualValue = Cross(RandomParameter.ActualValue, ParentsParameter.ActualValue);
      return base.Apply();
    }

    protected abstract DecisionList Cross(IRandom random, ItemArray<DecisionList> parents);
  }
}
